import java.sql.*;
import java.io.*;
import oracle.jdbc.OracleResultSet;
import oracle.sql.*;

/* Demonstrates writing Blobs using streams and locators
 * and reading Blobs using streams.
 */

public class BlobExamples
{
	static Connection conn;
	static final String IMAGE_FILE = "sunset.jpg";
	static final int BUFFER_SIZE = 2048;

	public static void main(String[] args)
	{
		try
		{
			Class.forName("oracle.jdbc.OracleDriver");
			conn =
				DriverManager.getConnection(
					"jdbc:oracle:oci:@osiris",
					"david",
					"bigcat");
			// make sure there is a suitable table
			Statement stmt = conn.createStatement();

			String sql = "DROP TABLE BLOB_TEST";
			try
			{
				stmt.execute(sql);
			}
			catch(SQLException e)
			{
			}

			sql =
				"CREATE TABLE BLOB_TEST (ID NUMBER, "
					+ "FILE_NAME VARCHAR2(128),"
					+ "IMAGE BLOB)";
			stmt.execute(sql);

			writeStream(100);
			writeLocator(200);
			readStream(100, "new100.jpg");
			readStream(200, "new200.jpg");			
			conn.close();
		}
		catch (Exception e)
		{
			System.out.println("Caught: " + e);
		}
	}

	public static void writeStream(int key) throws SQLException, IOException
	{
		// write blob using PreparedStatement setBinaryStream
		PreparedStatement ps;
		ps =
			conn.prepareStatement(
				"INSERT INTO BLOB_TEST (ID, FILE_NAME, IMAGE) " + "VALUES(?, ? , ?)");

		ps.setLong(1, key);
		ps.setString(2, IMAGE_FILE);

		File file = new File(IMAGE_FILE);
		FileInputStream fileStream = new FileInputStream(file);
		long length = file.length();
		ps.setBinaryStream(3, fileStream, (int) file.length());
		int i = ps.executeUpdate(); // i should be 1
		ps.close();
		fileStream.close();

	}

	public static void writeLocator(int key) throws SQLException, IOException
	{
		// Insert record with empty BLOB
		PreparedStatement ps =
			conn.prepareStatement(
				"INSERT INTO BLOB_TEST (ID, FILE_NAME, IMAGE) "
					+ "VALUES(?, ? , EMPTY_BLOB())");
		ps.setInt(1, key);

		ps.setString(2, IMAGE_FILE);
		ps.executeUpdate();
		ps.close();

		//	Get BLOB locator from record
		conn.setAutoCommit(false);
		ps =
			conn.prepareStatement(
				"SELECT IMAGE FROM BLOB_TEST " + "WHERE ID = ? FOR UPDATE");
		ps.setInt(1, key);
		ResultSet rs = ps.executeQuery();
		BLOB blob = null;
		if (rs.next())
		{
			blob = ((OracleResultSet) rs).getBLOB(1);
		}

		if (blob != null)
		{
			// Use locator to get stream
			//   Standard JDBC Blob method not supported by Oracle API:
			//   OutputStream outStream = blob.setBinaryStream(); 
			//   This is Oracle's version:
			OutputStream outStream = blob.getBinaryOutputStream(0);

			// use buffer to write from file to db
			File file = new File(IMAGE_FILE);
			FileInputStream inStream = new FileInputStream(file);
			int size = blob.getBufferSize();
			byte[] buffer = new byte[size];
			int length = -1;
			while ((length = inStream.read(buffer)) != -1)
			{
				outStream.write(buffer, 0, length);
			}
			inStream.close();
			outStream.close();
		}
		conn.commit();

		ps.close();
		rs.close();
	}

	public static void readStream(int key, String destination) throws SQLException, IOException
	{
		//	Read BLOB into file
		String sql = "SELECT FILE_NAME, IMAGE FROM BLOB_TEST " + "WHERE ID = ?";
		PreparedStatement ps = conn.prepareStatement(sql);
		ps.setInt(1, key);
		ResultSet rs = ps.executeQuery();
		if (rs.next())
		{
			// String filename = rs.getString(1); // Disregard and
			File file = new File(destination);// use "destination" instead
			FileOutputStream outStream = new FileOutputStream(file);
			InputStream inStream = rs.getBinaryStream(2);

			// set up buffer for transfer
			byte[] buffer = new byte[BUFFER_SIZE];
			int length = -1;

			// Read and write, buffer by buffer
			while ((length = inStream.read(buffer)) != -1)
			{
				outStream.write(buffer, 0, length);
			}
			inStream.close();
			outStream.close();
		}
		rs.close();
		ps.close();
	}
}
